"use client";
import useShowToast from "@/app/hooks/useShowToast";
import { editAttempt, storeAttempt } from "@/app/lib/attempt/action";
import { createKycRequest } from "@/app/lib/kyc/action";
import {
  ImageWrapper,
  InputWrapper,
  SelectWrapper,
} from "@/components/fields/Fields";
import FormComponent from "@/components/form/Form";
import { getStorage, removeStorage, setStorage } from "@/utils/navigation";
import prepareValidationSchema from "@/utils/validation-schema";
import {
  Box,
  Button,
  FormControl,
  FormLabel,
  Text,
  useColorModeValue,
} from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useEffect, useRef, useState } from "react";
import { formFields } from "../variables/formFields";
import WebcamCapture from "./WebcamCapture";
import {
  generateUniqueId,
  onCompleteImage,
  onUploadFiles,
} from "@/utils/common";
import useNetworkStatus from "@/app/hooks/useNetworkStatus";
import { ImageUpload } from "@/types/attachment";
import { AttemptedInterface } from "../../assignments/type/attempted";
import { createLocationRequest } from "@/app/lib/attendance/action";
import { getAssignmentList } from "@/app/lib/assignment/action";
import { BOOLEAN_OPTIONS, KYC_PREMISSIS_OPTIONS, KYC_PROPERTY_OPTIONS, METER_REQ_OPTIONS } from "../../dc/variables/options";
import ConditionalFieldPremisesOther from "@/components/fields/ConditionalFieldPremisesOther";

interface Location {
  latitude: string;
  longitude: string;
}

const KycForm = ({ data, draft = false }: { data?: any; draft?: boolean }) => {
  const formRef = useRef(null);
  const isOnline = useNetworkStatus();
  const router = useRouter();
  const showToast = useShowToast();
  const textColor = useColorModeValue("navy.700", "white");
  const brandStars = useColorModeValue("brand.500", "brand.400");
  const [isCamera, setIsCamera] = useState(false);
  const [imageSrc, setImageSrc] = useState([]);
  const [uploadImages, setUploadImages] = useState<ImageUpload[]>([]);
  const [loading, setLoading] = useState(false);
  const [location, setLocation] = useState<Location>({
    latitude: "",
    longitude: "",
  });

  const { initialFormData, validationSchema } =
    prepareValidationSchema(formFields);

  const getLocation = () => {
    if (navigator.geolocation) {
      navigator.geolocation.getCurrentPosition(
        (position: GeolocationPosition) => {
          setLocation({
            latitude: position.coords.latitude.toString(),
            longitude: position.coords.longitude.toString(),
          });
        },
        () => showToast("Failed", "Unable to retrieve location", "error")
      );
    } else {
      showToast(
        "Failed",
        "Geolocation is not supported by this browser",
        "error"
      );
    }
  };

  const handleAttemptUpdate = async (assignment_id: number, kyc_id: number) => {
    try {
      const response = await editAttempt(assignment_id, {
        kyc_id: kyc_id,
      });

      if (response) {
        removeStorage("kyc_update");
        router.push("/officer/assignments");
      }
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
    }
  };

  const handleCreateAttempt = async () => {
    try {
      const payload = draft
        ? {
            assignment_id: data?.assignment_id,
            case_id: data.case_id,
            batch_id: data.batch_id,
            consumer_id: null,
            dc_form_id: null,
            kyc_id: null,
          }
        : await getStorage("kyc");
      const response = await storeAttempt({ ...payload });
      return response;
    } catch (error: any) {
      showToast("Failed to create", error.message, "error");
      return null;
    }
  };

  interface FormData {
    [key: string]: any;
  }

  const handleCreateKycOffline = async (values: FormData) => {
    setLoading(true);
    try {
      const storedData = await getStorage("draft-kycs");
      const data = await getStorage("kyc_update");
      const newId = generateUniqueId();
      const consumersData = [
        ...(storedData || []),
        {
          ...values,
          created_at: new Date(),
          updated_at: new Date(),
          case_id: data.case_id,
          batch_id: data.batch_id,
          assignment_id: data.assignment_id,
          case: data?.case,
          offline: true,
          files: imageSrc,
          id: newId,
        },
      ];
      setStorage("draft-kycs", structuredClone(consumersData));
      removeStorage("kyc_update");
      router.push("/officer/kycs-drafts");
      showToast("Success", "Data Saved Locally", "success");
    } catch (error: any) {
      showToast("Failed", "Failed to Create Locally", "error");
    } finally {
      setLoading(false);
    }
  };

  const handleCreateKyc = async (
    values: FormData,
    attempt: AttemptedInterface
  ) => {
    setLoading(true);
    try {
      let uploadedData;

      if (draft && uploadImages?.length > 0 && data?.files?.length > 0) {
        const uploadImagesResult = await onUploadFiles(uploadImages);
        const dataFilesResult = await onCompleteImage(data?.files);
        uploadedData = [...uploadImagesResult, ...dataFilesResult];
      } else if (draft && uploadImages?.length > 0) {
        uploadedData = await onUploadFiles(uploadImages);
      } else if (imageSrc?.length > 0) {
        uploadedData = await onCompleteImage(imageSrc);
      } else if (draft && data?.files?.length > 0) {
        uploadedData = await onCompleteImage(data?.files);
      } else {
        uploadedData = [];
      }

      const response = await createKycRequest({
        ...values,
        files: uploadedData?.length > 0 ? uploadedData : [],
      });
      if (response) {
        removeStorage("kyc");
        if (attempt) await handleAttemptUpdate(attempt?.id, response?.data?.id);
        if (attempt)
          await createLocationRequest({
            latitude: values?.latitude,
            longitude: values?.longitude,
            status: "attempting",
            attempt_id: attempt?.id,
          });
        if (draft) {
          const storedData = await getStorage("draft-kycs");
          const newData = storedData.filter((val: any) => {
            return val.assignment_id !== attempt?.assignment_id;
          });
          setStorage("draft-kycs", structuredClone(newData));
        }
        const res = await getAssignmentList({ pagination: "false" });
        const updatedData = {
          last_sync_at: new Date().toISOString(),
          data: res?.data || [],
        };
        setStorage("assignments", updatedData);
        showToast("Success", response?.message, "success");
      }
    } catch (error: any) {
      showToast("Failed to create Kyc", error.message, "error");
    } finally {
      setLoading(false);
    }
  };

  const handleSubmit = async (values: FormData) => {
    const localData = await getStorage("lat-lng");
    values.latitude =
      location?.latitude?.toString() || localData?.latitude?.toString();
    values.longitude =
      location?.longitude?.toString() || localData?.longitude?.toString();

    if (draft) {
      values.latitude = data?.latitude;
      values.longitude = data?.longitude;
    }

    if (values?.id) {
      delete values.id;
    }

    if (isOnline) {
      if (!values?.latitude || !values?.longitude) {
        getLocation();
      }
      const response = await handleCreateAttempt();
      response?.data && handleCreateKyc(values, response?.data);
    } else {
      !data && handleCreateKycOffline(values);
    }
  };

  const handleUploadImages = async (images: ImageUpload[]) => {
    setUploadImages(images);
  };

  useEffect(() => {
    getLocation();
  }, []);

  return (
    <>
      <FormComponent
        ref={formRef}
        validationSchema={validationSchema}
        initialValues={data ? { ...data } : initialFormData}
        // enableReinitialize // Ensures the form reinitializes with updated initial values
        submitForm={handleSubmit}
      >
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Full Name &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="full_name"
              id="full_name"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.full_name ?? ""}
              disabled={!!data?.full_name}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Contact Information &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="contact_information"
              id="contact_information"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.contact_information ?? ""}
              disabled={!!data?.contact_information}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w="100%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Address &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="address"
              id="address"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.address ?? ""}
              disabled={!!data?.address}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              GIS Tagging 
            </FormLabel>
            <InputWrapper
              form
              fieldName="gis_tagging"
              id="gis_tagging"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.gis_tagging ?? ""}
              disabled={!!data?.gis_tagging}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Contract No. / Consumer No.
            </FormLabel>
            <InputWrapper
              form
              fieldName="contract_no_or_consumer_no"
              id="contract_no_or_consumer_no"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.contract_no_or_consumer_no ?? ""}
              disabled={!!data?.contract_no_or_consumer_no}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Meter No &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <InputWrapper
              form
              fieldName="meter_no"
              id="meter_no"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.meter_no ?? ""}
              disabled={!!data?.meter_no}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Other Meter Number &nbsp;
            </FormLabel>
            <InputWrapper
              form
              fieldName="other_meter_number"
              id="other_meter_number"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.other_meter_number ?? ""}
              disabled={!!data?.other_meter_number}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Premises in use: &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="premises_in_use"
              id="dc_request"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              options={BOOLEAN_OPTIONS}
              defaultValue={data?.premises_in_use ?? ""}
              disabled={!!data?.premises_in_use}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Meter exist on site: &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="meter_exist_on_site"
              id="meter_exist_on_site"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              fontWeight="500"
              options={BOOLEAN_OPTIONS}
              defaultValue={data?.meter_exist_on_site ?? ""}
              disabled={!!data?.meter_exist_on_site}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Premises Type &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="premises_type"
              id="premises_type"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              fontWeight="500"
              options={KYC_PREMISSIS_OPTIONS}
              defaultValue={data?.premises_type ?? ""}
              disabled={!!data?.premises_type}
            />
          </FormControl>
          <ConditionalFieldPremisesOther
            textColor={textColor}
            brandStars={brandStars}
            data={data}
          />
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Property Type &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="property_type"
              id="property_type"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              fontWeight="500"
              options={KYC_PROPERTY_OPTIONS}
              defaultValue={data?.property_type ?? ""}
              disabled={!!data?.property_type}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Meter Installed &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="meters_installed"
              id="meters_installed"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              fontWeight="500"
              options={BOOLEAN_OPTIONS}
              defaultValue={data?.meters_installed ?? ""}
              disabled={!!data?.meters_installed}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w="50%">
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Additional Meter &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="additional_meter"
              id="additional_meter"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              options={BOOLEAN_OPTIONS}
              defaultValue={data?.additional_meter ?? ""}
              disabled={!!data?.additional_meter}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Additional Meter Req: &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="additional_meter_req"
              id="additional_meter_req"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              fontWeight="500"
              options={METER_REQ_OPTIONS}
              defaultValue={data?.additional_meter_req ?? ""}
              disabled={!!data?.additional_meter_req}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Additional Meter Applied: &nbsp;
              <Text display="inline-block" color={brandStars}>
                *
              </Text>
            </FormLabel>
            <SelectWrapper
              form
              fieldName="additional_meter_applied"
              id="additional_meter_applied"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              fontWeight="500"
              options={BOOLEAN_OPTIONS}
              defaultValue={data?.additional_meter_applied ?? ""}
              disabled={!!data?.additional_meter_applied}
            />
          </FormControl>
        </Box>
        <Box display="flex" flexDir={{base: 'column', lg: 'row'}} gap={{base: 0, lg: 15}}>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Billing Dispute (if any)
            </FormLabel>
            <InputWrapper
              form
              fieldName="billing_dispute"
              id="billing_dispute"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.billing_dispute ?? ""}
              disabled={!!data?.billing_dispute}
            />
          </FormControl>
          <FormControl w={{base: "full", lg: "50%"}}>
            <FormLabel
              fontSize="sm"
              fontWeight="500"
              color={textColor}
              mb="8px"
            >
              Theft with Detail (if any)
            </FormLabel>
            <InputWrapper
              form
              fieldName="theft_detail"
              id="theft_detail"
              variant="auth"
              w="full"
              mb="2px"
              fontSize="sm"
              type="text"
              fontWeight="500"
              defaultValue={data?.theft_detail ?? ""}
              disabled={!!data?.theft_detail}
            />
          </FormControl>
        </Box>

        {isCamera && (
          <WebcamCapture
            setImageSrc={setImageSrc}
            onCaptureComplete={() => setIsCamera(false)}
          />
        )}

        {!data && (
          <Button onClick={() => setIsCamera(true)} mb={3}>
            Upload Photo
          </Button>
        )}

        {data && (
          <ImageWrapper
            form={false}
            updateOnParent={(images) => {
              handleUploadImages(images);
            }}
            label="Upload From Gallery"
            id="filess"
            fieldName="filess"
          />
        )}

        {imageSrc?.length > 0 && (
          <Box display="flex" flexWrap="wrap" width="100%" gap={4}>
            {imageSrc.map((src, index) => (
              <>
                <img
                  key={index}
                  src={src}
                  alt={`Captured ${index}`}
                  style={{
                    width: "150px",
                    height: "120px",
                    objectFit: "cover",
                    borderRadius: 10,
                  }}
                />
              </>
            ))}
          </Box>
        )}
        {data?.files?.length > 0 && (
          <Box display="flex" flexWrap="wrap" gap={4}>
            {data?.files?.map((src: string, index: number) => (
              <img
                key={index}
                src={src}
                alt={`Captured ${index}`}
                style={{
                  width: "150px",
                  height: "120px",
                  objectFit: "cover",
                  borderRadius: 10,
                }}
              />
            ))}
          </Box>
        )}

        <Button
          variant="brand"
          onClick={() => formRef?.current?.externalHandleSubmit()}
          isDisabled={loading}
          isLoading={loading}
        >
          Create
        </Button>
      </FormComponent>
    </>
  );
};

export default KycForm;
